# NOTE: this file cannot be in a class. See: https://github.com/pallets/click/issues/601
# https://click.palletsprojects.com/en/8.1.x/commands/#nested-handling-and-contexts

from typing import List, Tuple

import click

# Commands are imported when the function is run, to increase performance.
from todotree.Managers.TaskManager import TaskManager
from todotree.Config.Config import Config
from todotree.MainUtils import MainUtils


@click.group()
@MainUtils.common_options
def root(ctx: click.Context, **kwargs):
    """
    The main list of todotree's command.

    For more information on a command, use `todotree COMMAND --help`.
    """
    # ^ This text also shows up in the help command.
    # Parse options given before the command.
    # So the `--todo-file task.txt` option in the help text example.
    ctx.ensure_object(dict)
    ctx.obj["config"] = Config()
    MainUtils.parse_common_options(ctx.obj["config"], **kwargs)


@root.command("add", short_help="Add a task to the task list")
@click.argument("task", type=str, nargs=-1)
@MainUtils.common_options
def add(ctx: click.Context, task: Tuple, **kwargs):
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Add import Add

    Add(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task)


@root.command("addx", short_help="Add a task and immediately mark it as done")
@click.argument("task", type=str, nargs=-1)
@MainUtils.common_options
def add_x(ctx: click.Context, task: Tuple, **kwargs):
    """
    Adds a completed task to done.txt. The task is not added to todo.txt.
    :param task: The task to add to done.txt.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.AddX import AddX

    AddX(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task)


@root.command("append", short_help="Append `append_string` to `task_nr`")
@click.argument("task_nr", type=int)
@click.argument("append_string", type=str, nargs=-1)
@MainUtils.common_options
def append(ctx: click.Context, task_nr: int, append_string: Tuple[str], **kwargs):
    """
    Appends the contents of append_string to the task represented by task_nr.
    A space is inserted between the two tasks, so you do not have to worry that words aretogether.

    Example: todotree append 1 "some text"
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Append import Append

    Append(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task_nr, append_string)


@root.command("cd", short_help="Print directory of the todo.txt directory")
@MainUtils.common_options
def cd(ctx: click.Context, **kwargs):
    """print directory of the todo.txt directory"""
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Cd import Cd

    Cd(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("context", short_help="List task in a tree, by context")
@MainUtils.common_options
def context(ctx: click.Context, **kwargs):
    """list a tree, of which the first node is the context, the second nodes are the tasks"""
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Context import Context

    Context(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("do", short_help="Mark task as done and move it to the done.txt")
@click.argument("task_numbers", type=list, nargs=-1)  # type=list[int]
@MainUtils.common_options
def do(ctx: click.Context, task_numbers: List[Tuple], **kwargs):
    """
    Mark tasks as done, therefor moving them to done.txt with a date stamp of today.
    :param task_numbers: The list of tasks which are completed.
    """

    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Do import Do

    Do(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task_numbers)


@root.command("due", short_help="List tasks by their due date")
@MainUtils.common_options
def due(ctx: click.Context, **kwargs):
    """List tasks in a tree by their due date."""
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Due import Due

    Due(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("edit", short_help="Open the todo.txt in an editor.")
@MainUtils.common_options
def edit(ctx: click.Context, **kwargs):
    """
    Open the todo.txt in an editor for manual editing of tasks.
    This is useful when you need to modify a lot of tasks, which would be complicated when doing it with todotree.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Edit import Edit

    Edit(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("filter", short_help="Only show tasks containing the search term.")
@click.argument("search_term")
@MainUtils.common_options
def filter_list(ctx: click.Context, search_term, **kwargs):
    """
    Only show tasks which have search term in them. This can also be a keyword.

    :param search_term: The term to search.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Filter import Filter

    Filter(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(search_term)


@root.command("help", short_help="Print help text.")
@click.pass_context
def help_text(ctx: click.Context):
    print(ctx.parent.get_help())


@root.command("init", short_help="Initialize folder for first use.")
@MainUtils.common_options
def init(ctx: click.Context, **kwargs):
    """
    Initializes the files and folders needed for a functioning todotree according to the user's prompts.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Init import Init

    Init().run()


@root.command("list", short_help="List tasks")
@MainUtils.common_options
def list_tasks(ctx: click.Context, **kwargs):
    """
    Print a flat list of tasks, sorted by their priority.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.List import List

    MainUtils.parse_common_options(ctx.obj["config"], **kwargs)
    List(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("list_done", short_help="List tasks which are marked as done")
@MainUtils.common_options
def list_done(ctx: click.Context, **kwargs):
    """
    List tasks which are marked as done. The numbers can be used with the revive command.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.ListDone import ListDone

    ListDone(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("print_raw", short_help="Print todo.txt without any formatting or filtering")
@MainUtils.common_options
def print_raw(ctx: click.Context, **kwargs):
    """
    Output the todo.txt without any processing.
    This is equivalent to `cat $(todo cd)/todo.txt` in bash.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.PrintRaw import PrintRaw

    PrintRaw(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("priority", short_help="Set new priority to task")
@click.argument("task_number", type=int)
@click.argument("new_priority", type=str)
@MainUtils.common_options
def priority(ctx: click.Context, task_number, new_priority, **kwargs):
    """
    Adds or updates the priority of the task.
    :param task_number: The task to re-prioritize.
    :param new_priority: The new priority.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Priority import Priority

    Priority(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task_number, new_priority)


@root.command("project", short_help="Print tree by project")
@MainUtils.common_options
def project(ctx: click.Context, **kwargs):
    """
    Print the task list in a tree by project.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Project import Project

    Project(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("recur", short_help="Add recurring tasks defined in recur.txt")
@MainUtils.common_options
def recur(ctx: click.Context, **kwargs):
    """
    Add tasks according to recur.txt.

    In recur.txt you can add tasks that have to be done periodically, such as doing taxes.
    The format in recur.txt is:

        `<start date> ; interval ; <end date> ; <task string>`

    The end date is optional. This is a valid line for example:
    2020-01-01 ; yearly ; (C) Celebrate the new year.

    This is certainly not the only program which can do these.
    Different solutions to this command exist, here are a few alternatives:
     - https://www.rememberthemilk.com/
     - https://github.com/smartlitchi/ice-recur
    You can use those with the `addons` command after installing a suitable addon script.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Recur import Recur

    Recur(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("revive", short_help="Revive a task that was accidentally marked as done.")
@click.argument("done_number", type=int)
@MainUtils.common_options
def revive(ctx: click.Context, done_number, **kwargs):
    """
    Move a task from done.txt to todo.txt.
    The `done_number` can be found using the `list_done` command.
    :param done_number: The number of the task to revive.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Revive import Revive

    Revive(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(done_number)


@root.command("schedule", short_help="Hide task until date.")
@click.argument("task_number", type=int)
@click.argument("new_date", type=str, nargs=-1)
@MainUtils.common_options
def schedule(ctx: click.Context, task_number: int, new_date: Tuple[str], **kwargs):
    """
    hide the task until the date given. If new_date is not in ISO format (yyyy-mm-dd) such as "Next Wednesday",
    then it tries to figure out the date with the `date` program, which is only in linux.
    """
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Schedule import Schedule

    Schedule(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(new_date, task_number)


@root.group("stale", short_help="Manage the stale.txt task list.")
@MainUtils.common_options
def stale(ctx: click.Context, **kwargs):
    # process for stale.txt
    pass


@stale.command("list", short_help="List the stale tasks.")
@MainUtils.common_options
def stale_list(ctx: click.Context, **kwargs):
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Stale_list import StaleList
    StaleList(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@stale.command("revive", short_help="Move task from stale.txt to todo.txt")
@click.argument("task_number", type=int)
@MainUtils.common_options
def stale_revive(ctx: click.Context, task_number: int, **kwargs):
    """Remove task with index `task_number` from stale.txt and adds it back to todo.txt."""
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Stale_revive import StaleRevive
    StaleRevive(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task_number)


@stale.command("add", short_help="Move task from todo.txt to stale.txt")
@click.argument("task_number", type=int)
@MainUtils.common_options
def stale_add(ctx: click.Context, task_number: int, **kwargs):
    """Removes the task with the index `task_number` from todo.txt and adds it to stale.txt."""
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Stale_add import StaleAdd
    StaleAdd(ctx.obj["config"], TaskManager(ctx.obj["config"])).run(task_number)


@root.command("version", short_help="Show the version of TodoTree.")
@MainUtils.common_options
def version(ctx: click.Context, **kwargs):
    MainUtils.initialize(ctx.obj["config"], **kwargs)
    from todotree.Commands.Version import Version

    Version(ctx.obj["config"], TaskManager(ctx.obj["config"])).run()


@root.command("addons", short_help="Run an addon script")
@click.argument("command", type=str)
@MainUtils.common_options
def addons_command(ctx: click.Context, command: str, **kwargs):
    """
    Run an addon script.
    The script must be in the addons_folder. It can be any language you like: It does not have to be python.
    However, it must have the executable bit set.

    :param command: The script/command to run.
    """
    from todotree.Addons import Addons

    MainUtils.initialize(ctx.obj["config"], **kwargs)
    result = Addons(ctx.obj["config"]).run(command)
    click.echo(result.stdout, nl=False)
    if result.returncode != 0:
        click.echo(result.stderr)
    ctx.obj["config"].git.commit_and_push(f"addons {command}")


#  End Region Command Definitions.
#  Setup Click

CONTEXT_SETTINGS: dict = dict(help_option_names=["-h", "--help"])
"""Click context settings. See https://click.palletsprojects.com/en/8.1.x/complex/ for more information."""
cli: click.CommandCollection = click.CommandCollection(sources=[root], context_settings=CONTEXT_SETTINGS)
"""Command Collection defining defaults. https://click.palletsprojects.com/en/8.1.x/api/#click.CommandCollection ."""

if __name__ == "__main__":
    cli(obj={})



import datetime
import re
from itertools import chain
from typing import Optional

from todotree.Config.Config import Config
from todotree.Errors.RecurParseError import RecurParseError
from todotree.Task.RecurTask import RecurTask
from todotree.Managers.TaskManager import TaskManager


class RecurManager:
    """Manages the recur.txt tasks."""

    comment_regex = re.compile(r"^\s*#")
    """Regular expression to detect comments."""

    recur_timestamp_filename = ".recur.timestamp"
    """Name of the timestamp file."""

    def __init__(self, config: Config):
        self.config: Config = config
        self.task_list: list[RecurTask] = []
        self.task_manager: TaskManager = TaskManager(self.config)
        self.last_date: Optional[datetime.date] = None

    def import_tasks(self):
        """Import the tasks from recur.txt."""
        error = False
        with self.config.paths.recur_file.open("r") as f:
            for line_number, recur_task in enumerate(f.readlines()):
                if not recur_task.strip():
                    # Empty line, continue.
                    continue
                if self.comment_regex.match(recur_task):
                    # Comment, continue.
                    continue
                try:
                    task = RecurTask(recur_task)
                    self.task_list.append(task)
                except RecurParseError as e:
                    self.config.console.error(f"Error parsing Recur on line {line_number}.")
                    self.config.console.error(str(e))
                    error = True
                    continue
                if task.end_date < task.start_date:
                    self.config.console.warning(f"Recur task on line {line_number} is ending before it is starting.")
                    self.config.console.warning("This task will never get added.")
        if error:
            raise RecurParseError("One or more tasks were invalid, aborted adding recur tasks.")
        # Also import the tasks itself to compare later.
        self.task_manager.import_tasks()

    def add_to_todo(self):
        """Add the imported tasks to todo.txt"""
        # Check time of last time recur ran.
        self.check_last_time_run()
        # If already run, just exit.
        if self.last_date == datetime.date.today():
            self.config.console.info("Recur has already run today.")
            return

        # Filter the tasks that need to be added.
        self.filter_by_date()
        self.filter_by_existing()
        # Add the remaining Recur Tasks to the task list.
        prev, added_tasks = self.task_manager.add_tasks_to_file([x.to_task() for x in self.task_list])
        if added_tasks:
            self.config.console.info("Added the following tasks:")
        else:
            self.config.console.info("No recurring task to add.")
        for i, task in enumerate(added_tasks):
            self.config.console.info(f"{task}")

    def filter_by_date(self):
        """Removes tasks that are not allowed to be added because of their date information."""
        new_list = []
        for task in self.task_list:
            if task.start_date > datetime.date.today():
                # Recur task has not started yet.
                continue
            if task.end_date < datetime.date.today():
                # Recur task has ended.
                continue
            match task.interval:
                case RecurTask.Recurrence.Never:
                    if self.last_date < task.start_date < datetime.date.today():
                        new_list.append(task)
                    elif self.last_date < task.end_date < datetime.date.today():
                        new_list.append(task)
                case RecurTask.Recurrence.Daily:
                    # Then it should always be added.
                    new_list.append(task)
                case RecurTask.Recurrence.Weekly:
                    self.__match_week(new_list, task, datetime.date.today())
                case RecurTask.Recurrence.Monthly:
                    self.__match_month(new_list, task, datetime.date.today())
                case RecurTask.Recurrence.Yearly:
                    self.__match_year(new_list, task, datetime.date.today())
        # End loop
        self.task_list = new_list

    def __match_week(self, new_list, task, today):
        # Count up until the week is further than last date.
        date = task.start_date
        while date < self.last_date:
            date = date + datetime.timedelta(days=7)
        if date < today:
            new_list.append(task)

    def __match_month(self, new_list, task, today):
        if self.last_date.day < today.day:
            # case: 12 13 14 15
            if task.start_date.day in range(self.last_date.day, today.day):
                new_list.append(task)
        else:
            # case: 30 31 1 2 3 4 5
            if task.start_date.day in chain(range(self.last_date.day, 31), range(1, today.day)):
                new_list.append(task)

    def __match_year(self, new_list, task, today):
        if self.last_date.year < today.year:
            # case: 2022 2023
            if task.start_date.month in range(self.last_date.month):
                # Check in last year
                if self.last_date < datetime.date(self.last_date.year, task.start_date.month,
                                                  task.start_date.day) < today:
                    new_list.append(task)
            else:
                # Check in new year.
                if self.last_date < datetime.date(today.year, task.start_date.month, task.start_date.day) < today:
                    new_list.append(task)
        else:
            # Both in same year.
            if self.last_date < datetime.date(self.last_date.year, task.start_date.month, task.start_date.day) < today:
                new_list.append(task)

    def filter_by_existing(self):
        """Removes tasks that already exist in todo.txt"""
        existing_tasks = [x.task_string for x in self.task_manager.task_list]
        self.task_list = [item for item in self.task_list if item.task_string not in existing_tasks]

    def check_last_time_run(self):
        if self.last_date is not None:
            # Then it was already checked.
            return
        timestamp_file = self.config.paths.todo_folder / self.recur_timestamp_filename
        if not timestamp_file.exists():
            # Then run from the beginning of time.
            self.last_date = datetime.date(1, 1, 1)
            return
        timestamp = timestamp_file.stat().st_mtime
        self.last_date = datetime.date.fromtimestamp(timestamp)

    def set_last_time_run(self):
        timestamp_file = self.config.paths.todo_folder / self.recur_timestamp_filename
        if not timestamp_file.exists():
            timestamp_file.touch()
        # This changes the mtime.
        timestamp_file.write_text("The modified time of this file is used by `recur`, please do not modify it.")




from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.RecurParseError import RecurParseError
from todotree.Managers.RecurManager import RecurManager


class Recur(AbstractCommand):

    def run(self):
        if not self.config.paths.recur_file.exists():
            self.config.console.error("recur.txt not found, nothing to add.")
            self.config.console.error("It should be at the following location:")
            self.config.console.error(self.config.paths.recur_file)
            exit(1)
        rm = RecurManager(self.config)
        try:
            rm.import_tasks()
        except RecurParseError as e:
            self.config.console.error(str(e))
            exit(1)
        rm.add_to_todo()
        rm.set_last_time_run()
        self.config.git.commit_and_push("recur")



from abc import ABC

from todotree.Config.Config import Config
from todotree.Managers.TaskManager import TaskManager


class AbstractCommand(ABC):

    def __init__(self, config: Config, task_manager: TaskManager):
        """
        Initializes a new AbstractCommand.
        :param config: The configuration of the application.
        :param task_manager: The task manager.
        """
        self.taskManager = task_manager
        self.config = config




import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError


class List(AbstractCommand):
    def run(self):
        try:
            self.taskManager.import_tasks()
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)

        self.config.console.info("Todos")
        click.echo(self.taskManager)



from typing import Tuple, List

import click
from dateutil.utils import today

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.DoneFileNotFoundError import DoneFileNotFoundError
from todotree.Errors.NoSuchTaskError import NoSuchTaskError
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError
from todotree.Managers.DoneManager import DoneManager
from todotree.Task.DoneTask import DoneTask


class Do(AbstractCommand):

    def run(self, task_numbers: List[Tuple]):
        # Convert to ints. Task numbers is a list of tuples. Each tuple contains one digit of the number.
        new_numbers: List[int] = []
        for task_tuple in task_numbers:
            new_number: str = ""
            for task_digit in task_tuple:
                new_number += task_digit
            new_numbers.append(int(new_number))
        # Write back to old value.
        task_numbers = new_numbers
        # Marking something as Done cannot be done with fancy imports
        # So we disable them.
        self.config.enable_project_folder = False
        try:
            self.taskManager.import_tasks()
            completed_tasks = self.taskManager.remove_tasks_from_file(task_numbers)
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)
            exit(1)  # IDE hinting.
        except NoSuchTaskError as e:
            self.config.console.error(e.message)
            exit(1)
        try:
            done_manager = DoneManager(self.config)
            done_manager.import_tasks()
            done_manager.add_tasks_to_file([DoneTask(x.i, f"x {today().date()} {x.task_string}") for x in completed_tasks])
        except DoneFileNotFoundError as e:
            e.echo_and_exit(self.config)
            exit(1)  # For IDE hinting.
        # Print the results
        self.config.console.info("Tasks marked as done:")
        for task in completed_tasks:
            click.echo(task)
        self.config.git.commit_and_push("do")



import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError


class Context(AbstractCommand):

    def run(self):
        try:
            self.taskManager.import_tasks()
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)
        click.echo(self.taskManager.print_tree("contexts"))



import click

from todotree.Commands.AbstractCommand import AbstractCommand


class Edit(AbstractCommand):
    def run(self):
        # Disable fancy imports.
        self.config.enable_project_folder = False
        click.edit(filename=str(self.config.paths.todo_file))
        self.config.git.commit_and_push("edit")



import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError


class Due(AbstractCommand):
    def run(self):
        # Disable fancy imports, because they do not have due dates.
        self.config.enable_project_folder = False
        # Import tasks.
        try:
            self.taskManager.import_tasks()
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)
        # Print due tree.
        click.echo(self.taskManager.print_tree("due"))



from typing import Tuple

import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError
from todotree.Task.Task import Task


class Add(AbstractCommand):

    def run(self, task: Tuple):
        # Convert tuple to string
        task: str = " ".join(map(str, task))
        try:
            # Disable fancy imports, because they are not needed.
            self.config.enable_project_folder = False
            # Import tasks.
            self.taskManager.import_tasks()
            # Add task
            new_number = self.taskManager.add_tasks_to_file([Task(0, task)])
            self.config.console.info("Task added:")
            click.echo(f"{new_number} {task}")
            self.config.git.commit_and_push("add")
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)



from typing import Tuple

import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.NoSuchTaskError import NoSuchTaskError
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError


class Append(AbstractCommand):
    def run(self, task_nr: int, append_string: Tuple[str]):
        # Disable fancy imports, because they are not needed.
        self.config.enable_project_folder = False
        # Import tasks.
        try:
            self.taskManager.import_tasks()
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)
        # Convert tuple to string.
        append_string = " ".join(append_string)
        # Append task.
        try:
            new_task = self.taskManager.append_to_task(task_nr, append_string.strip())
        except NoSuchTaskError as e:
            self.config.console.error(e.message)
            exit(1)
        self.config.console.info("The new task is: ")
        click.echo(new_task)
        self.config.git.commit_and_push("append")



from typing import Tuple

import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.DoneFileNotFoundError import DoneFileNotFoundError
from todotree.Task.DoneTask import DoneTask


class AddX(AbstractCommand):

    def run(self, task: Tuple):
        done = DoneTask.task_to_done(" ".join(map(str, task)))
        if not self.config.paths.done_file.exists():
            DoneFileNotFoundError("").echo_and_exit(self.config)
        with self.config.paths.done_file.open("a") as f:
            f.write(done)
        click.echo(done)



import click

from todotree.Commands.AbstractCommand import AbstractCommand
from todotree.Errors.DoneFileNotFoundError import DoneFileNotFoundError
from todotree.Errors.NoSuchTaskError import NoSuchTaskError
from todotree.Errors.TodoFileNotFoundError import TodoFileNotFoundError
from todotree.Managers.DoneManager import DoneManager
from todotree.Task.Task import Task


class Revive(AbstractCommand):
    def run(self, done_number: int):
        done_manager = DoneManager(self.config)
        try:
            done_manager.import_tasks()
            revived_task = done_manager.remove_task_from_file(done_number)
        except DoneFileNotFoundError as e:
            e.echo_and_exit(self.config)
            exit(1)  # Never reached.
        except NoSuchTaskError as e:
            self.config.console.error(e.message)
            exit(1)
        try:
            self.taskManager.import_tasks()
            self.taskManager.add_tasks_to_file([Task(revived_task.i, revived_task.task_string)])
        except TodoFileNotFoundError as e:
            e.echo_and_exit(self.config)
        self.config.git.commit_and_push("revive")



import subprocess
from pathlib import Path

from todotree.Config.Config import Config


# Addons https://click.palletsprojects.com/en/8.1.x/commands/#custom-multi-commands

class Addons:

    def __init__(self, config: Config = Config()):
        self.rv: list = []
        """List of programs."""

        self.default_config: Config = config
        """Configuration"""

    def run(self, name):
        """Run the addon with the name `name`."""
        file = Path(self.default_config.paths.addons_folder / name)
        if not file.exists():
            self.default_config.console.error(f"There is no script at {file}")
            exit(1)

        self.default_config.console.verbose(f"Running script at {file}")
        try:
            result = subprocess.run([file, str(self.default_config.paths.todo_file)], capture_output=True, text=True)
        except Exception as e:
            self.default_config.console.error(f"Error while running the addon {name}.")
            self.default_config.console.error(f"The error is {e}")
            raise e
        return result



class TodotreeError(Exception):
    """
    Generic Exception class for application specific exceptions.
    """
    def __init__(self, message=""):
        self.message = message

        super().__init__(message)

    def __str__(self):
        return self.message
